--                                                                    --
--  package Py.Load_Python_Library  Copyright (c)  Dmitry A. Kazakov  --
--  Implementation                                 Luebeck            --
--                                                 Winter, 2018       --
--                                                                    --
--                                Last revision :  13:09 10 Mar 2013  --
--                                                                    --
--  This  library  is  free software; you can redistribute it and/or  --
--  modify it under the terms of the GNU General Public  License  as  --
--  published by the Free Software Foundation; either version  2  of  --
--  the License, or (at your option) any later version. This library  --
--  is distributed in the hope that it will be useful,  but  WITHOUT  --
--  ANY   WARRANTY;   without   even   the   implied   warranty   of  --
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU  --
--  General  Public  License  for  more  details.  You  should  have  --
--  received  a  copy  of  the GNU General Public License along with  --
--  this library; if not, write to  the  Free  Software  Foundation,  --
--  Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.    --
--                                                                    --
--  As a special exception, if other files instantiate generics from  --
--  this unit, or you link this unit with other files to produce  an  --
--  executable, this unit does not by  itself  cause  the  resulting  --
--  executable to be covered by the GNU General Public License. This  --
--  exception  does not however invalidate any other reasons why the  --
--  executable file might be covered by the GNU Public License.       --
--____________________________________________________________________--

with Ada.IO_Exceptions;           use Ada.IO_Exceptions;
with Strings_Edit;                use Strings_Edit;
with Strings_Edit.Quoted;         use Strings_Edit.Quoted;
with Strings_Edit.UTF8.Handling;  use Strings_Edit.UTF8.Handling;

package body Py.Load_Python_Library is

      -- Lazy function call binding
   RTLD_LAZY         : constant := 16#00001#;
      -- Immediate function call binding
   RTLD_NOW          : constant := 16#00002#;
      -- Mask of binding time value
   RTLD_BINDING_MASK : constant := 16#00003#;
      -- Do not load the object
   TLD_NOLOAD        : constant := 16#00004#;
      -- Use deep binding
   RTLD_DEEPBIND     : constant := 16#00008#;
      -- If the following  bit is set  in the MODE argument to `dlopen',
      -- the symbols of the loaded object and its dependencies  are made
      -- visible as if the object were linked directly into the program.
   RTLD_GLOBAL       : constant := 16#00100#;
      -- Unix98 demands  the following  flag  which  is  the  inverse to
      -- RTLD_GLOBAL. The implementation  does this by default and so we
      -- can define the value to zero.
   RTLD_LOCAL        : constant := 16#0#;
      -- Do not delete object when closed
   RTLD_NODELETE     : constant := 16#01000#;

   function dlopen
            (  Name  : char_array;
               Flags : int := RTLD_NOW + RTLD_GLOBAL
            )  return Address;
   pragma Import (C, dlopen);

   function dlsym
            (  Module : Address;
               Name   : char_array := "Py_AddPendingCall" & Nul
            )  return AddPendingCall_Ptr;
   function dlsym
            (  Module : Address;
               Name   : char_array := "PyBytes_AsStringAndSize" & Nul
            )  return Bytes_AsStringAndSize_Ptr;
   function dlsym
            (  Module : Address;
               Name   : char_array := "PyCallable_Check" & Nul
            )  return Callable_Check_Ptr;
   function dlsym
            (  Module : Address;
               Name   : char_array := "Py_DecRef" & Nul
            )  return DecRef_Ptr;
   function dlsym
            (  Module : Address;
               Name   : char_array := "PyDict_GetItemString" & Nul
            )  return Dict_GetItemString_Ptr;
   function dlsym
            (  Module : Address;
               Name   : char_array := "PyDict_Keys" & Nul
            )  return Dict_Keys_Ptr;
   function dlsym
            (  Module : Address;
               Name   : char_array := "PyDict_Items" & Nul
            )  return Dict_Items_Ptr;
   function dlsym
            (  Module : Address;
               Name   : char_array := "PyDict_New" & Nul
            )  return Dict_New_Ptr;
   function dlsym
            (  Module : Address;
               Name   : char_array := "PyDict_SetItemString" & Nul
            )  return Dict_SetItemString_Ptr;
   function dlsym
            (  Module : Address;
               Name   : char_array := "PyDict_Size" & Nul
            )  return Dict_Size_Ptr;
   function dlsym
            (  Module : Address;
               Name   : char_array := "PyErr_BadArgument" & Nul
            )  return Err_BadArgument_Ptr;
   function dlsym
            (  Module : Address;
               Name   : char_array := "PyErr_BadInternalCall" & Nul
            )  return Err_BadInternalCall_Ptr;
   function dlsym
            (  Module : Address;
               Name   : char_array := "PyErr_Clear" & Nul
            )  return Err_Clear_Ptr;
   function dlsym
            (  Module : Address;
               Name   : char_array := "PyErr_Fetch" & Nul
            )  return Err_Fetch_Ptr;
   function dlsym
            (  Module : Address;
               Name   : char_array := "PyErr_NewException" & Nul
            )  return Err_NewException_Ptr;
   function dlsym
            (  Module : Address;
               Name   : char_array := "PyErr_NormalizeException" & Nul
            )  return Err_NormalizeException_Ptr;
   function dlsym
            (  Module : Address;
               Name   : char_array := "PyErr_Occurred" & Nul
            )  return Err_Occurred_Ptr;
   function dlsym
            (  Module : Address;
               Name   : char_array := "PyErr_Restore" & Nul
            )  return Err_Restore_Ptr;
   function dlsym
            (  Module : Address;
               Name   : char_array := "PyErr_SetString" & Nul
            )  return Err_SetString_Ptr;
   function dlsym
            (  Module : Address;
               Name   : char_array := "PyEval_InitThreads" & Nul
            )  return Eval_InitThreads_Ptr;
   function dlsym
            (  Module : Address;
               Name   : char_array := "Py_FinalizeEx" & Nul
            )  return FinalizeEx_Ptr;
   function dlsym
            (  Module : Address;
               Name   : char_array := "PyFloat_AsDouble" & Nul
            )  return Float_AsDouble_Ptr;
   function dlsym
            (  Module : Address;
               Name   : char_array := "PyFloat_FromDouble" & Nul
            )  return Float_FromDouble_Ptr;
   function dlsym
            (  Module : Address;
               Name   : char_array := "PyGILState_Ensure" & Nul
            )  return GILState_Ensure_Ptr;
   function dlsym
            (  Module : Address;
               Name   : char_array := "PyGILState_Release" & Nul
            )  return GILState_Release_Ptr;
   function dlsym
            (  Module : Address;
               Name   : char_array := "PyImport_AppendInittab" & Nul
            )  return Import_AppendInittab_Ptr;
   function dlsym
            (  Module : Address;
               Name   : char_array := "Py_IncRef" & Nul
            )  return IncRef_Ptr;
   function dlsym
            (  Module : Address;
               Name   : char_array := "Py_InitializeEx" & Nul
            )  return InitializeEx_Ptr;
   function dlsym
            (  Module : Address;
               Name   : char_array := "PyImport_Import" & Nul
            )  return Import_Import_Ptr;
   function dlsym
            (  Module : Address;
               Name   : char_array := "PyImport_ImportModule" & Nul
            )  return Import_ImportModule_Ptr;
   function dlsym
            (  Module : Address;
               Name   : char_array := "PyList_Append" & Nul
            )  return List_Append_Ptr;
   function dlsym
            (  Module : Address;
               Name   : char_array := "PyList_GetItem" & Nul
            )  return List_GetItem_Ptr;
   function dlsym
            (  Module : Address;
               Name   : char_array := "PyList_Insert" & Nul
            )  return List_Insert_Ptr;
   function dlsym
            (  Module : Address;
               Name   : char_array := "PyList_New" & Nul
            )  return List_New_Ptr;
   function dlsym
            (  Module : Address;
               Name   : char_array := "PyList_Size" & Nul
            )  return List_Size_Ptr;
   function dlsym
            (  Module : Address;
               Name   : char_array := "PyLong_AsLong" & Nul
            )  return Long_AsLong_Ptr;
   function dlsym
            (  Module : Address;
               Name   : char_array := "PyLong_FromLong" & Nul
            )  return Long_FromLong_Ptr;
   function dlsym
            (  Module : Address;
               Name   : char_array := "PyModule_Create2" & Nul
            )  return Module_Create_Ptr;
   function dlsym
            (  Module : Address;
               Name   : char_array := "PyModule_GetDict" & Nul
            )  return Module_GetDict_Ptr;
   function dlsym
            (  Module : Address;
               Name   : char_array := "PyObject_CallObject" & Nul
            )  return Object_CallObject_Ptr;
   function dlsym
            (  Module : Address;
               Name   : char_array := "PyObject_GetAttrString" & Nul
            )  return Object_GetAttrString_Ptr;
   function dlsym
            (  Module : Address;
               Name   : char_array := "PyObject_Dir" & Nul
            )  return Object_Dir_Ptr;
   function dlsym
            (  Module : Address;
               Name   : char_array := "PyObject_Str" & Nul
            )  return Object_Str_Ptr;
   function dlsym
            (  Module : Address;
               Name   : char_array := "PySequence_DelItem" & Nul
            )  return Sequence_DelItem_Ptr;
   function dlsym
            (  Module : Address;
               Name   : char_array := "PySequence_Index" & Nul
            )  return Sequence_Index_Ptr;
   function dlsym
            (  Module : Address;
               Name   : char_array := "PySys_GetObject" & Nul
            )  return Sys_GetObject_Ptr;
   function dlsym
            (  Module : Address;
               Name   : char_array := "PyTuple_GetItem" & Nul
            )  return Tuple_GetItem_Ptr;
   function dlsym
            (  Module : Address;
               Name   : char_array := "PyTuple_New" & Nul
            )  return Tuple_New_Ptr;
   function dlsym
            (  Module : Address;
               Name   : char_array := "PyTuple_SetItem" & Nul
            )  return Tuple_SetItem_Ptr;
   function dlsym
            (  Module : Address;
               Name   : char_array := "PyTuple_Size" & Nul
            )  return Tuple_Size_Ptr;
   function dlsym
            (  Module : Address;
               Name   : char_array := "PyUnicode_AsEncodedString" & Nul
            )  return Unicode_AsEncodedString_Ptr;
   function dlsym
            (  Module : Address;
               Name   : char_array := "PyUnicode_DecodeFSDefault" & Nul
            )  return Unicode_DecodeFSDefault_Ptr;
   function dlsym
            (  Module : Address;
               Name   : char_array := "PyUnicode_FromString" & Nul
            )  return Unicode_FromString_Ptr;
   function dlsym
            (  Module : Address;
               Name   : char_array := "PyUnicode_GetLength" & Nul
            )  return Unicode_GetLength_Ptr;
   function dlsym
            (  Module : Address;
               Name   : char_array := "PyUnicode_ReadChar" & Nul
            )  return Unicode_ReadChar_Ptr;

   function dlsym
            (  Module : Address;
               Name   : char_array
            )  return Object;
   function dlsym
            (  Module : Address;
               Name   : char_array
            )  return Get_Object_Ptr;
   pragma Import (C, dlsym);

   Library : Address := Null_Address;
   Loaded  : Boolean := False;

   function Get_Default_Name return String is
   begin
      return "libpython3.6m.so.1";
   end Get_Default_Name;

   function Get_Extension return String is
   begin
      return "*.so.*";
   end Get_Extension;

   function Is_Loaded return Boolean is
   begin
      return Loaded;
   end Is_Loaded;

   procedure Load (Name : String := "") is
      function Get_Library_Path return String is
      begin
         if Name'Length = 0 then
            return Get_Default_Name;
         else
            return Trim (Name);
         end if;
      end Get_Library_Path;
   begin
      if Loaded then
         return;
      end if;
      declare
         Name : constant String := Get_Library_Path;
      begin
         if Library = Null_Address then
            Library := dlopen (To_C (Name));
            if Library = Null_Address then
               Raise_Exception
               (  Use_Error'Identity,
                  "Failed to load Python DLL " & Quote (Name)
               );
            end if;
         end if;
         Links.AddPendingCall          := dlsym (Library);
         Links.Bytes_AsStringAndSize   := dlsym (Library);
         Links.Callable_Check          := dlsym (Library);
         Links.DecRef                  := dlsym (Library);
         Links.Dict_GetItemString      := dlsym (Library);
         Links.Dict_Items              := dlsym (Library);
         Links.Dict_Keys               := dlsym (Library);
         Links.Dict_New                := dlsym (Library);
         Links.Dict_SetItemString      := dlsym (Library);
         Links.Dict_Size               := dlsym (Library);
         Links.Err_BadArgument         := dlsym (Library);
         Links.Err_BadInternalCall     := dlsym (Library);
         Links.Err_Clear               := dlsym (Library);
         Links.Err_Fetch               := dlsym (Library);
         Links.Err_NewException        := dlsym (Library);
         Links.Err_NormalizeException  := dlsym (Library);
         Links.Err_Occurred            := dlsym (Library);
         Links.Err_Restore             := dlsym (Library);
         Links.Err_SetString           := dlsym (Library);
         Links.Eval_InitThreads        := dlsym (Library);
         Links.Exc_KeyError            := dlsym
                                          (  Library,
                                             "PyExc_KeyError" & Nul
                                          );
         Links.Exc_RuntimeError        := dlsym
                                          (  Library,
                                             "PyExc_RuntimeError" & Nul
                                          );
         Links.Exc_SystemError         := dlsym
                                          (  Library,
                                             "PyExc_SystemError" & Nul
                                          );
         Links.Exc_TypeError           := dlsym
                                          (  Library,
                                             "PyExc_TypeError" & Nul
                                          );
         Links.Exc_ValueError          := dlsym
                                          (  Library,
                                             "PyExc_ValueError" & Nul
                                          );
         Links.False                   := dlsym
                                          (  Library,
                                             "_Py_FalseStruct" & Nul
                                          );
         Links.FinalizeEx              := dlsym (Library);
         Links.Float_AsDouble          := dlsym (Library);
         Links.Float_FromDouble        := dlsym (Library);
         Links.GILState_Ensure         := dlsym (Library);
         Links.GILState_Release        := dlsym (Library);
         Links.Import_AppendInittab    := dlsym (Library);
         Links.Import_Import           := dlsym (Library);
         Links.Import_ImportModule     := dlsym (Library);
         Links.IncRef                  := dlsym (Library);
         Links.InitializeEx            := dlsym (Library);
         Links.List_Append             := dlsym (Library);
         Links.List_GetItem            := dlsym (Library);
         Links.List_Insert             := dlsym (Library);
         Links.List_New                := dlsym (Library);
         Links.List_Size               := dlsym (Library);
         Links.Long_AsLong             := dlsym (Library);
         Links.Long_FromLong           := dlsym (Library);
         Links.Module_Create           := dlsym (Library);
         Links.Module_GetDict          := dlsym (Library);
         Links.None                    := dlsym
                                          (  Library,
                                             "_Py_NoneStruct" & Nul
                                          );
         Links.Object_CallObject       := dlsym (Library);
         Links.Object_Dir              := dlsym (Library);
         Links.Object_GetAttrString    := dlsym (Library);
         Links.Object_Str              := dlsym (Library);
         Links.Sequence_DelItem        := dlsym (Library);
         Links.Sequence_Index          := dlsym (Library);
         Links.Sys_GetObject           := dlsym (Library);
         Links.True                    := dlsym
                                          (  Library,
                                             "_Py_TrueStruct" & Nul
                                          );
         Links.Tuple_GetItem           := dlsym (Library);
         Links.Tuple_New               := dlsym (Library);
         Links.Tuple_SetItem           := dlsym (Library);
         Links.Tuple_Size              := dlsym (Library);
         Links.Unicode_AsEncodedString := dlsym (Library);
         Links.Unicode_DecodeFSDefault := dlsym (Library);
         Links.Unicode_FromString      := dlsym (Library);
         Links.Unicode_GetLength       := dlsym (Library);
         Links.Unicode_ReadChar        := dlsym (Library);

         Check_Links (Name);
      end;
      Loaded := True;
   end Load;

   pragma Linker_Options ("-ldl");

end Py.Load_Python_Library;
