--                                                                    --
--  package Py.Load_Python_Library  Copyright (c)  Dmitry A. Kazakov  --
--  Implementation                                 Luebeck            --
--                                                 Winter, 2018       --
--                                                                    --
--                                Last revision :  13:09 10 Mar 2013  --
--                                                                    --
--  This  library  is  free software; you can redistribute it and/or  --
--  modify it under the terms of the GNU General Public  License  as  --
--  published by the Free Software Foundation; either version  2  of  --
--  the License, or (at your option) any later version. This library  --
--  is distributed in the hope that it will be useful,  but  WITHOUT  --
--  ANY   WARRANTY;   without   even   the   implied   warranty   of  --
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU  --
--  General  Public  License  for  more  details.  You  should  have  --
--  received  a  copy  of  the GNU General Public License along with  --
--  this library; if not, write to  the  Free  Software  Foundation,  --
--  Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.    --
--                                                                    --
--  As a special exception, if other files instantiate generics from  --
--  this unit, or you link this unit with other files to produce  an  --
--  executable, this unit does not by  itself  cause  the  resulting  --
--  executable to be covered by the GNU General Public License. This  --
--  exception  does not however invalidate any other reasons why the  --
--  executable file might be covered by the GNU Public License.       --
--____________________________________________________________________--

with Ada.IO_Exceptions;           use Ada.IO_Exceptions;
with Strings_Edit;                use Strings_Edit;
with Strings_Edit.Integers;       use Strings_Edit.Integers;
with Strings_Edit.Quoted;         use Strings_Edit.Quoted;
with Strings_Edit.UTF8.Handling;  use Strings_Edit.UTF8.Handling;

with Ada.Streams.Stream_IO;
with Interfaces;

package body Py.Load_Python_Library is

   LOAD_WITH_ALTERED_SEARCH_PATH       : constant := 16#00000008#;
   LOAD_LIBRARY_SEARCH_DLL_LOAD_DIR    : constant := 16#00000100#;
   LOAD_LIBRARY_SEARCH_APPLICATION_DIR : constant := 16#00000200#;
   LOAD_LIBRARY_SEARCH_DEFAULT_DIRS    : constant := 16#00001000#;

   ERROR_PROC_NOT_FOUND : constant := 127;
   ERROR_BAD_EXE_FORMAT : constant := 193;

   type DWORD  is new unsigned_long;

   function LoadLibrary
            (  Name  : wchar_array;
               File  : Address := Null_Address;
               Flags : DWORD   := LOAD_WITH_ALTERED_SEARCH_PATH
            )  return Address;
   pragma Import (Stdcall, LoadLibrary, "LoadLibraryExW");

   function GetProcAddress
            (  Module : Address;
               Name   : char_array := "Py_AddPendingCall" & Nul
            )  return AddPendingCall_Ptr;
   function GetProcAddress
            (  Module : Address;
               Name   : char_array := "PyBytes_AsStringAndSize" & Nul
            )  return Bytes_AsStringAndSize_Ptr;
   function GetProcAddress
            (  Module : Address;
               Name   : char_array := "PyCallable_Check" & Nul
            )  return Callable_Check_Ptr;
   function GetProcAddress
            (  Module : Address;
               Name   : char_array := "Py_DecRef" & Nul
            )  return DecRef_Ptr;
   function GetProcAddress
            (  Module : Address;
               Name   : char_array := "PyDict_GetItemString" & Nul
            )  return Dict_GetItemString_Ptr;
   function GetProcAddress
            (  Module : Address;
               Name   : char_array := "PyDict_Keys" & Nul
            )  return Dict_Keys_Ptr;
   function GetProcAddress
            (  Module : Address;
               Name   : char_array := "PyDict_Items" & Nul
            )  return Dict_Items_Ptr;
   function GetProcAddress
            (  Module : Address;
               Name   : char_array := "PyDict_New" & Nul
            )  return Dict_New_Ptr;
   function GetProcAddress
            (  Module : Address;
               Name   : char_array := "PyDict_SetItemString" & Nul
            )  return Dict_SetItemString_Ptr;
   function GetProcAddress
            (  Module : Address;
               Name   : char_array := "PyDict_Size" & Nul
            )  return Dict_Size_Ptr;
   function GetProcAddress
            (  Module : Address;
               Name   : char_array := "PyErr_BadArgument" & Nul
            )  return Err_BadArgument_Ptr;
   function GetProcAddress
            (  Module : Address;
               Name   : char_array := "PyErr_BadInternalCall" & Nul
            )  return Err_BadInternalCall_Ptr;
   function GetProcAddress
            (  Module : Address;
               Name   : char_array := "PyErr_Clear" & Nul
            )  return Err_Clear_Ptr;
   function GetProcAddress
            (  Module : Address;
               Name   : char_array := "PyErr_Fetch" & Nul
            )  return Err_Fetch_Ptr;
   function GetProcAddress
            (  Module : Address;
               Name   : char_array := "PyErr_NewException" & Nul
            )  return Err_NewException_Ptr;
   function GetProcAddress
            (  Module : Address;
               Name   : char_array := "PyErr_NormalizeException" & Nul
            )  return Err_NormalizeException_Ptr;
   function GetProcAddress
            (  Module : Address;
               Name   : char_array := "PyErr_Occurred" & Nul
            )  return Err_Occurred_Ptr;
   function GetProcAddress
            (  Module : Address;
               Name   : char_array := "PyErr_Restore" & Nul
            )  return Err_Restore_Ptr;
   function GetProcAddress
            (  Module : Address;
               Name   : char_array := "PyErr_SetString" & Nul
            )  return Err_SetString_Ptr;
   function GetProcAddress
            (  Module : Address;
               Name   : char_array := "PyEval_InitThreads" & Nul
            )  return Eval_InitThreads_Ptr;
   function GetProcAddress
            (  Module : Address;
               Name   : char_array := "Py_FinalizeEx" & Nul
            )  return FinalizeEx_Ptr;
   function GetProcAddress
            (  Module : Address;
               Name   : char_array := "PyFloat_AsDouble" & Nul
            )  return Float_AsDouble_Ptr;
   function GetProcAddress
            (  Module : Address;
               Name   : char_array := "PyFloat_FromDouble" & Nul
            )  return Float_FromDouble_Ptr;
   function GetProcAddress
            (  Module : Address;
               Name   : char_array := "PyGILState_Ensure" & Nul
            )  return GILState_Ensure_Ptr;
   function GetProcAddress
            (  Module : Address;
               Name   : char_array := "PyGILState_Release" & Nul
            )  return GILState_Release_Ptr;
   function GetProcAddress
            (  Module : Address;
               Name   : char_array := "PyImport_AppendInittab" & Nul
            )  return Import_AppendInittab_Ptr;
   function GetProcAddress
            (  Module : Address;
               Name   : char_array := "Py_IncRef" & Nul
            )  return IncRef_Ptr;
   function GetProcAddress
            (  Module : Address;
               Name   : char_array := "Py_InitializeEx" & Nul
            )  return InitializeEx_Ptr;
   function GetProcAddress
            (  Module : Address;
               Name   : char_array := "PyImport_Import" & Nul
            )  return Import_Import_Ptr;
   function GetProcAddress
            (  Module : Address;
               Name   : char_array := "PyImport_ImportModule" & Nul
            )  return Import_ImportModule_Ptr;
   function GetProcAddress
            (  Module : Address;
               Name   : char_array := "PyList_Append" & Nul
            )  return List_Append_Ptr;
   function GetProcAddress
            (  Module : Address;
               Name   : char_array := "PyList_GetItem" & Nul
            )  return List_GetItem_Ptr;
   function GetProcAddress
            (  Module : Address;
               Name   : char_array := "PyList_Insert" & Nul
            )  return List_Insert_Ptr;
   function GetProcAddress
            (  Module : Address;
               Name   : char_array := "PyList_New" & Nul
            )  return List_New_Ptr;
   function GetProcAddress
            (  Module : Address;
               Name   : char_array := "PyList_Size" & Nul
            )  return List_Size_Ptr;
   function GetProcAddress
            (  Module : Address;
               Name   : char_array := "PyLong_AsLong" & Nul
            )  return Long_AsLong_Ptr;
   function GetProcAddress
            (  Module : Address;
               Name   : char_array := "PyLong_FromLong" & Nul
            )  return Long_FromLong_Ptr;
   function GetProcAddress
            (  Module : Address;
               Name   : char_array := "PyModule_Create2" & Nul
            )  return Module_Create_Ptr;
   function GetProcAddress
            (  Module : Address;
               Name   : char_array := "PyModule_GetDict" & Nul
            )  return Module_GetDict_Ptr;
   function GetProcAddress
            (  Module : Address;
               Name   : char_array := "PyObject_CallObject" & Nul
            )  return Object_CallObject_Ptr;
   function GetProcAddress
            (  Module : Address;
               Name   : char_array := "PyObject_Dir" & Nul
            )  return Object_Dir_Ptr;
   function GetProcAddress
            (  Module : Address;
               Name   : char_array := "PyObject_GetAttrString" & Nul
            )  return Object_GetAttrString_Ptr;
   function GetProcAddress
            (  Module : Address;
               Name   : char_array := "PyObject_Str" & Nul
            )  return Object_Str_Ptr;
   function GetProcAddress
            (  Module : Address;
               Name   : char_array := "PySequence_DelItem" & Nul
            )  return Sequence_DelItem_Ptr;
   function GetProcAddress
            (  Module : Address;
               Name   : char_array := "PySequence_Index" & Nul
            )  return Sequence_Index_Ptr;
   function GetProcAddress
            (  Module : Address;
               Name   : char_array := "PySys_GetObject" & Nul
            )  return Sys_GetObject_Ptr;
   function GetProcAddress
            (  Module : Address;
               Name   : char_array := "PyTuple_GetItem" & Nul
            )  return Tuple_GetItem_Ptr;
   function GetProcAddress
            (  Module : Address;
               Name   : char_array := "PyTuple_New" & Nul
            )  return Tuple_New_Ptr;
   function GetProcAddress
            (  Module : Address;
               Name   : char_array := "PyTuple_SetItem" & Nul
            )  return Tuple_SetItem_Ptr;
   function GetProcAddress
            (  Module : Address;
               Name   : char_array := "PyTuple_Size" & Nul
            )  return Tuple_Size_Ptr;
   function GetProcAddress
            (  Module : Address;
               Name   : char_array := "PyUnicode_AsEncodedString" & Nul
            )  return Unicode_AsEncodedString_Ptr;
   function GetProcAddress
            (  Module : Address;
               Name   : char_array := "PyUnicode_DecodeFSDefault" & Nul
            )  return Unicode_DecodeFSDefault_Ptr;
   function GetProcAddress
            (  Module : Address;
               Name   : char_array := "PyUnicode_FromString" & Nul
            )  return Unicode_FromString_Ptr;
   function GetProcAddress
            (  Module : Address;
               Name   : char_array := "PyUnicode_GetLength" & Nul
            )  return Unicode_GetLength_Ptr;
   function GetProcAddress
            (  Module : Address;
               Name   : char_array := "PyUnicode_ReadChar" & Nul
            )  return Unicode_ReadChar_Ptr;

   function GetProcAddress
            (  Module : Address;
               Name   : char_array
            )  return Object;
   function GetProcAddress
            (  Module : Address;
               Name   : char_array
            )  return Get_Object_Ptr;
   pragma Import (Stdcall, GetProcAddress, "GetProcAddress");

   function GetLastError return DWORD;
   pragma Import (Stdcall, GetLastError, "GetLastError");

   Library : Address := Null_Address;
   Loaded  : Boolean := False;

   function Get_Default_Name return String is
   begin
      return "python3.dll";
   end Get_Default_Name;

   function Get_Extension return String is
   begin
      return "*.dll";
   end Get_Extension;

   function Is_Loaded return Boolean is
   begin
      return Loaded;
   end Is_Loaded;

   type Machine_Type is new Interfaces.Unsigned_16;
   MACHINE_UNKNOWN   : constant Machine_Type := 16#0000#;
   MACHINE_AM33      : constant Machine_Type := 16#01d3#;
   MACHINE_AMD64     : constant Machine_Type := 16#8664#;
   MACHINE_ARM       : constant Machine_Type := 16#01c0#;
   MACHINE_EBC       : constant Machine_Type := 16#0ebc#;
   MACHINE_I386      : constant Machine_Type := 16#014c#;
   MACHINE_IA64      : constant Machine_Type := 16#0200#;
   MACHINE_M32R      : constant Machine_Type := 16#9041#;
   MACHINE_MIPS16    : constant Machine_Type := 16#0266#;
   MACHINE_MIPSFPU   : constant Machine_Type := 16#0366#;
   MACHINE_MIPSFPU16 : constant Machine_Type := 16#0466#;
   MACHINE_POWERPC   : constant Machine_Type := 16#01f0#;
   MACHINE_POWERPCFP : constant Machine_Type := 16#01f1#;
   MACHINE_R4000     : constant Machine_Type := 16#0166#;
   MACHINE_SH3       : constant Machine_Type := 16#01a2#;
   MACHINE_SH3DSP    : constant Machine_Type := 16#01a3#;
   MACHINE_SH4       : constant Machine_Type := 16#01a6#;
   MACHINE_SH5       : constant Machine_Type := 16#01a8#;
   MACHINE_THUMB     : constant Machine_Type := 16#01c2#;
   MACHINE_WCEMIPSV2 : constant Machine_Type := 16#0169#;

   function Get_Machine (Path : String) return Machine_Type is
      use Ada.Streams;
      use Ada.Streams.Stream_IO;
      use Interfaces;
      File   : File_Type;
      function Byte return Stream_Element is
         Data : Stream_Element_Array (1..1);
         Last : Stream_Element_Offset;
      begin
         Read (File, Data, Last);
         if Last = 1 then
            return Data (1);
         end if;
         raise Ada.Streams.Stream_IO.End_Error;
      end Byte;
      Offset : Unsigned_32;
      Result : Machine_Type := MACHINE_UNKNOWN;
   begin
      Open (File, In_File, Path, "b");
      begin
         Set_Index (File, 16#3c# + 1);
         Offset :=          Unsigned_32 (Byte);
         Offset := Offset + Unsigned_32 (Byte) * 2**8;
         Offset := Offset + Unsigned_32 (Byte) * 2**16;
         Offset := Offset + Unsigned_32 (Byte) * 2**24;
         Set_Index (File, Positive_Count (Offset) + 1);
         if (  Character'Pos ('P') = Byte
            and then
               Character'Pos ('E') = Byte
            and then
               0 = Byte
            and then
               0 = Byte
            )
         then
            Result :=          Machine_Type (Byte);
            Result := Result + Machine_Type (Byte) * 2**8;
         end if;
         Close (File);
      exception
         when others =>
            Close (File);
      end;
      return Result;
   exception
      when others =>
         Close (File);
         return Result;
   end Get_Machine;

   FORMAT_MESSAGE_ALLOCATE_BUFFER : constant := 16#0100#;
   FORMAT_MESSAGE_IGNORE_INSERTS  : constant := 16#0200#;
   FORMAT_MESSAGE_FROM_STRING     : constant := 16#0400#;
   FORMAT_MESSAGE_FROM_HMODULE    : constant := 16#0800#;
   FORMAT_MESSAGE_FROM_SYSTEM     : constant := 16#1000#;
   FORMAT_MESSAGE_ARGUMENT_ARRAY  : constant := 16#2000#;
   FORMAT_MESSAGE_MAX_WIDTH_MASK  : constant := 16#00FF#;

   function FormatMessage
            (  Flags      : DWORD := FORMAT_MESSAGE_FROM_SYSTEM or
                                     FORMAT_MESSAGE_IGNORE_INSERTS or
                                     FORMAT_MESSAGE_ARGUMENT_ARRAY;
               Source     : chars_ptr := Null_Ptr;
               MessageId  : DWORD;
               LanguageId : DWORD := 0;
               Buffer     : access char;
               Size       : DWORD;
               Arguments  : System.Address := Null_Address
            )  return DWORD;
   pragma Import (Stdcall, FormatMessage, "FormatMessageA");

   function GetErrorText (Error : DWORD) return String is
      Buffer : char_array (1..2048);
      Length : DWORD;
   begin
      Length := FormatMessage
                (  MessageId => Error,
                   Buffer    => Buffer (1)'Access,
                   Size      => Buffer'Length
                );
      declare
         Message : String (1..Natural (Length));
      begin
         for Index in 1..size_t (Length) loop
            Message (Positive (Index)) :=
               Character'Val (char'Pos (Buffer (Index)));
         end loop;
         for Index in reverse Message'Range loop
            case Message (Index) is
               when Character'Val (0)..' ' =>
                  null;
               when others =>
                  return Message (1..Index) &
                         " [" & Image (Integer (Error)) & "]";
            end case;
         end loop;
         return "[" & Image (Integer (Error)) & "]";
      end;
   end GetErrorText;

   procedure Raise_From_LastError
             (  ID     : Exception_ID;
                Prefix : String := "";
                Error  : DWORD
             )  is
   begin
      if Prefix'Length > 0 then
         Raise_Exception (ID, Prefix & GetErrorText (Error));
      else
         Raise_Exception (ID, GetErrorText (Error));
      end if;
   end Raise_From_LastError;

   procedure Load (Name : String := "") is
      Error : DWORD;
      function Get_Library_Path return String is
      begin
         if Name'Length = 0 then
            return Get_Default_Name;
         else
            return Trim (Name);
         end if;
      end Get_Library_Path;
   begin
      if Loaded then
         return;
      end if;
      declare
         use Interfaces;
         Name  : constant String := Get_Library_Path;
      begin
         if Library = Null_Address then
            Library := LoadLibrary (To_C (To_Wide_String (Name)));
            if Library = Null_Address then
               Error := GetLastError;
               if Error = ERROR_BAD_EXE_FORMAT then
                  case Get_Machine (Name) is
                     when MACHINE_AMD64 =>
                        if Address'Size = 32 then
                           Raise_Exception
                           (  Use_Error'Identity,
                              (  "The DLL "
                              &  Quote (Name)
                              &  " is for AMD64 64-bit Windows,"
                              &  " use a 32-bit Python version instead"
                           )  );
                        end if;
                     when MACHINE_I386 =>
                        if Address'Size = 64 then
                           Raise_Exception
                           (  Use_Error'Identity,
                              (  "The DLL "
                              &  Quote (Name)
                              &  " is for x86 32-bit Windows,"
                              &  " use a 64-bit Python version instead"
                           )  );
                        end if;
                     when others =>
                        null;
                  end case;
               end if;
               Raise_From_LastError
               (  Use_Error'Identity,
                  "Failed to load Python DLL " & Quote (Name) & ": ",
                  Error
               );
            end if;
         end if;
         Links.AddPendingCall          := GetProcAddress (Library);
         if Links.AddPendingCall = null then
            Error := GetLastError;
            if Error /= ERROR_PROC_NOT_FOUND then
               Raise_From_LastError
               (  Use_Error'Identity,
                  (  "Failed to load Python DLL "
                  &  Quote (Name)
                  &  ", when looking for entry points: "
                  ),
                  Error
               );
            end if;
         end if;
         Links.Bytes_AsStringAndSize   := GetProcAddress (Library);
         Links.Callable_Check          := GetProcAddress (Library);
         Links.DecRef                  := GetProcAddress (Library);
         Links.Dict_GetItemString      := GetProcAddress (Library);
         Links.Dict_Items              := GetProcAddress (Library);
         Links.Dict_Keys               := GetProcAddress (Library);
         Links.Dict_New                := GetProcAddress (Library);
         Links.Dict_SetItemString      := GetProcAddress (Library);
         Links.Dict_Size               := GetProcAddress (Library);
         Links.Err_BadArgument         := GetProcAddress (Library);
         Links.Err_BadInternalCall     := GetProcAddress (Library);
         Links.Err_Clear               := GetProcAddress (Library);
         Links.Err_Fetch               := GetProcAddress (Library);
         Links.Err_NewException        := GetProcAddress (Library);
         Links.Err_NormalizeException  := GetProcAddress (Library);
         Links.Err_Occurred            := GetProcAddress (Library);
         Links.Err_Restore             := GetProcAddress (Library);
         Links.Err_SetString           := GetProcAddress (Library);
         Links.Eval_InitThreads        := GetProcAddress (Library);
         Links.Exc_KeyError            := GetProcAddress
                                          (  Library,
                                             "PyExc_KeyError" & Nul
                                          );
         Links.Exc_RuntimeError        := GetProcAddress
                                          (  Library,
                                             "PyExc_RuntimeError" & Nul
                                          );
         Links.Exc_SystemError         := GetProcAddress
                                          (  Library,
                                             "PyExc_SystemError" & Nul
                                          );
         Links.Exc_TypeError           := GetProcAddress
                                          (  Library,
                                             "PyExc_TypeError" & Nul
                                          );
         Links.Exc_ValueError          := GetProcAddress
                                          (  Library,
                                             "PyExc_ValueError" & Nul
                                          );
         Links.False                   := GetProcAddress
                                          (  Library,
                                             "_Py_FalseStruct" & Nul
                                          );
         Links.FinalizeEx              := GetProcAddress (Library);
         Links.Float_AsDouble          := GetProcAddress (Library);
         Links.Float_FromDouble        := GetProcAddress (Library);
         Links.GILState_Ensure         := GetProcAddress (Library);
         Links.GILState_Release        := GetProcAddress (Library);
         Links.Import_AppendInittab    := GetProcAddress (Library);
         Links.Import_Import           := GetProcAddress (Library);
         Links.Import_ImportModule     := GetProcAddress (Library);
         Links.IncRef                  := GetProcAddress (Library);
         Links.InitializeEx            := GetProcAddress (Library);
         Links.List_Append             := GetProcAddress (Library);
         Links.List_GetItem            := GetProcAddress (Library);
         Links.List_Insert             := GetProcAddress (Library);
         Links.List_New                := GetProcAddress (Library);
         Links.List_Size               := GetProcAddress (Library);
         Links.Long_AsLong             := GetProcAddress (Library);
         Links.Long_FromLong           := GetProcAddress (Library);
         Links.Module_Create           := GetProcAddress (Library);
         Links.Module_GetDict          := GetProcAddress (Library);
         Links.None                    := GetProcAddress
                                          (  Library,
                                             "_Py_NoneStruct" & Nul
                                          );
         Links.Object_CallObject       := GetProcAddress (Library);
         Links.Object_Dir              := GetProcAddress (Library);
         Links.Object_GetAttrString    := GetProcAddress (Library);
         Links.Object_Str              := GetProcAddress (Library);
         Links.Sequence_DelItem        := GetProcAddress (Library);
         Links.Sequence_Index          := GetProcAddress (Library);
         Links.Sys_GetObject           := GetProcAddress (Library);
         Links.True                    := GetProcAddress
                                          (  Library,
                                             "_Py_TrueStruct" & Nul
                                          );
         Links.Tuple_GetItem           := GetProcAddress (Library);
         Links.Tuple_New               := GetProcAddress (Library);
         Links.Tuple_SetItem           := GetProcAddress (Library);
         Links.Tuple_Size              := GetProcAddress (Library);
         Links.Unicode_AsEncodedString := GetProcAddress (Library);
         Links.Unicode_DecodeFSDefault := GetProcAddress (Library);
         Links.Unicode_FromString      := GetProcAddress (Library);
         Links.Unicode_GetLength       := GetProcAddress (Library);
         Links.Unicode_ReadChar        := GetProcAddress (Library);

         Check_Links (Name);
      end;
      Loaded := True;
   end Load;
end Py.Load_Python_Library;
